/***************************************************************
Neuer Ansatz: Ufolandung Objektorientiert
Das ermöglicht das Bewegen der verschiedensten Objekte 
  Ufo muss gelandet werden
  "b" Vertikale Rakete
  "n" Rakete nach links
  "v" Rakete nach rechts
****************************************************************/

//-------- Globale Variablen
int XSIZE = 800;   // Breite des Fensters
int YSIZE = 600;   // Höhe des Fensters
long lastMillis;

ufo myUfo;                  // Varuiablen zur Speicherung der Objekte anlegen
platform thePlatform;
space theSpace;

// =============== Setup wird nur einmal am Anfang ausgeführt ==========================
void setup() {

  size(XSIZE, YSIZE);                       // Bildschimfenster erzeugen
  background(0);                            // Schwarzer Hintergrund
  
  theSpace = new space();
  thePlatform = new platform(random(50,XSIZE-50));  // Zufällige Anfangsposition der Plattform
  myUfo = new ufo(0.0, 0.0, random(50, XSIZE-50), float(YSIZE-40), 0.01, 300);   // Ausgangsposition des ufos

}
 
// =========== Draw wird 30mal pro Sekunde ausgeführt - das ist der GameLoop ============= 
void draw() {
  
  if (!gameOver()) {
    background(0);   // Bildschirm löschen
    if (millis() > lastMillis+2000) {
      theSpace.addSpaceObject();   // Alle 2 Sekunden wird ein neuer Komet generiert
      lastMillis=millis();
    }
    myUfo.show();
    thePlatform.show();
    spritAnzeige(myUfo.getFuel());
    myUfo.move();
    theSpace.update();
    
  }
}



// ====================== Funktionen ======================================================

void spritAnzeige(int wert) {
  stroke(255);
  strokeWeight(1);
  fill(0);
  rect(10,30,20,300);   // Äußeres Rechteck zeichnen
  fill(0,255,0);
  rect(10,330-wert,20,wert);
}

// ===== Spielende Check ====== 
// True - Spiel ist beendet
// False- Spiel geht weiter
// Das Spiel endet, wenn das Ufo einen der Ränder des Bildschirms überschreitet

boolean gameOver() {
  boolean state=true;
  float x,y;
  
  x = myUfo.getPos();
  y = myUfo.getHight();
  
  if ((x>XSIZE)||(x<0)) bad();
  else
  if (y>YSIZE) bad();
  else
  if ((y<10)&&(abs(x-thePlatform.getPos())>15)) bad();
  else
  if ((y<40)&&(abs(x-thePlatform.getPos())<15)&&(myUfo.getSpeedV()<2)) good();
  else
  if ((y<40)&&(abs(x-thePlatform.getPos())<15)&&(myUfo.getSpeedV()>=2)) bad();
  else
  if (theSpace.collision(x,y)==1) {
    myUfo.setFuel(myUfo.getFuel()+50);
    state=false;
  }
  else
  if (theSpace.collision(x,y)==-1) bad();
  else state=false;
  return(state); 
} 

// Gelbe "Korona" umd das Raumschiff zeichnen, wenn die Landung geglückt ist
void good() {
  strokeWeight(0);
  fill(255,255,0,40);
  ellipse(int(myUfo.getPos()), YSIZE-int(myUfo.getHight()), 70, 70);
  myUfo.show();
}

// Rotes Kreuz zeichnen, wenn die Landung nicht geglückt ist
void bad()
{
  stroke(255,0,0);
  strokeWeight(10);
  line(0,0,XSIZE,YSIZE);
  line(XSIZE,0,0,YSIZE);
}  

// Von Processing reservierte Call-Back Funktion, die immer dann aufgerufen wird, 
// wenn eine Taste gedrückt wird.

void keyPressed()
{
  if (myUfo.getFuel()>0) {
    myUfo.fireRocket(key); 
  }
  if (key==' ') setup();               // Space Taste = Neues Spiel
}

